<?php

namespace Drupal\flashpoint_access_code\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\flashpoint_access_code\Entity\FlashpointAccessCode;
use Drupal\flashpoint_access_code\Entity\FlashpointAccessCodeUsage;
use Drupal\group\Entity\Group;
use Drupal\user\Entity\User;

/**
 * Class FlashpointAccessCodeSettingsForm.
 *
 * @ingroup flashpoint_access_code
 */
class FlashpointAccessCodeRedemptionForm extends FormBase {

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'flashpoint_access_code_redemption';
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $code = $form_state->getValue('access_code');
    $group_id = $form_state->getValue('group_id');
    $redeemer = User::load(\Drupal::currentUser()->id());
    $query = \Drupal::entityQuery('flashpoint_access_code')
      ->condition('access_code', $code)
      ->condition('group_id', $group_id);
    $result = $query->execute();
    $grant_access = FALSE;
    $previous = FALSE;
    $acid = FALSE;
    $messenger = \Drupal::messenger();
    // If the access code doesn't exist, do not redeem
    if (!count($result)) {
      $messenger->addError(t('The access code <strong><em>%s</em></strong> was not found for this course.', ['%s' => $code]));
    }
    // Check that the person may use the code.
    else {
      $acid = array_shift($result);
      $access_code = FlashpointAccessCode::load($acid);

      $max = $access_code->getMaxUsage();
      $current = $access_code->getCurrentUsage();

      if (!$max || $current < $max) {
        $grant_access = TRUE;
      }
      else {
        $messenger->addError(t('The access code <strong><em>%s</em></strong> has been used the maximum number of times, and cannot be used for additional redemptions.', ['%s' => $code]));
      }

      // Double check that the user hasn't used the code already.
      if ($grant_access) {
        $previous = $access_code->getPreviousRedemption($redeemer->id());
        if ($previous) {
          $handler = \Drupal::config('flashpoint.settings')->get('flashpoint_access_code.previous_redemption');
          switch($handler) {
            case 'grant':
              $grant_access = TRUE;
              break;
            case 'grant_increment':
              $grant_access = TRUE;
              $previous = FALSE;
              break;
            case 'deny':
            default:
            $messenger->addError(t('You have used the access code <strong><em>%s</em></strong> previously, and may not re-use it.', ['%s' => $code]));
              $grant_access = FALSE;
              break;
          }
        }
      }
    }

    // If access is granted, then proceed with creating a membership.
    if ($grant_access) {
      $group = Group::load($group_id);
      $membership = $group->getMember($redeemer);
      if (!$membership) {
        $group->addMember($redeemer);
        if (!$previous) {
          $usage_values = [
            'user_id' => $redeemer->id(),
            'redeemer' => $redeemer->id(),
            'group_id' => $group_id,
            'access_code' => $acid,
            'status' => TRUE,
          ];
          $usage = FlashpointAccessCodeUsage::create($usage_values);
          $usage->save();
        }
        $messenger->addStatus(t('You have redeemed the access code <strong><em>%s</em></strong>.', ['%s' => $code]));
        $form_state->setRedirect('entity.group.canonical', ['group' => $group_id]);
      }
      else {
        $messenger->addError('You already have access ot this course. If you are having trouble with access, please contact the site administrator.');
      }
    }
  }

  /**
   * Defines the settings form for Flashpoint Access Code entities.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   Form definition array.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['access_code'] = ['#type' => 'textfield'];
    $form['access_code']['#title'] = 'Please enter the access code you have received for this course.';
    $group = \Drupal::routeMatch()->getParameter('group');
    $form['group_id'] = ['#type' => 'hidden', '#value' => $group->id()];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'Submit'
    ];
    return $form;
  }

}
